# Copyright 2017 Canonical Ltd.
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; version 3 of the License.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.


import dbus.service
import multiprocessing

from . import constants
from libertine import utils
from time import time


class Download(dbus.service.Object):
    def __init__(self, connection, id):
        self._finished = multiprocessing.Event()
        self._result = multiprocessing.SimpleQueue()
        self._error = multiprocessing.SimpleQueue()
        dbus.service.Object.__init__(self, conn=connection, object_path=(constants.DOWNLOAD_OBJECT % id))

    @property
    def id(self):
        return self._object_path

    @property
    def done(self):
        return self._finished.is_set()

    @property
    def result(self):
        if self._result.empty():
            return ''

        return self._result.get()

    @property
    def last_error(self):
        if self._error.empty():
            return ''

        return self._error.get()

    def data(self, message):
        self._result.put(message)

    # Signals to satisfy the download interface

    @dbus.service.signal(constants.DOWNLOAD_INTERFACE, signature='o')
    def processing(self, path):
        utils.get_logger().debug("emit processing('%s')" % path)

    @dbus.service.signal(constants.DOWNLOAD_INTERFACE, signature='o')
    def finished(self, path):
        utils.get_logger().debug("emit finished('%s')" % path)
        self._finished.set()

    @dbus.service.signal(constants.DOWNLOAD_INTERFACE)
    def progress(self, received, total):
        utils.get_logger().debug("emit progress(%d, %d)" % (received, total))

    @dbus.service.signal(constants.DOWNLOAD_INTERFACE, signature='s')
    def error(self, message):
        utils.get_logger().error("emit error(%s)" % message)
        self._error.put(message)
        self._finished.set()
