# Copyright 2017 Canonical Ltd.
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; version 3 of the License.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.


import multiprocessing
import os

from libertine import utils
from threading import Timer


class TaskDispatcherBase(object):
    def __init__(self, monitor, client):
        self._monitor = monitor
        self._client = client
        self._tasks = []

        self._delete_loop = 3
        if os.getenv("LIBERTINED_TESTING", False):
            self._delete_loop = 0.01

    def _cleanup(self):
        for task in [task for task in self._tasks if task.should_delete]:
            task.thread.join()
            self._monitor.remove_from_connection(task.id)
            self._tasks.remove(task)

        if len(self._tasks) > 0:
            Timer(self._delete_loop, self._cleanup).start()

    def _add_task(self, task):
        self._tasks.append(task)
        task.start()
        if len(self._tasks) == 1:
            Timer(self._delete_loop, self._cleanup).start()

        return task.id
