# Copyright 2016-2017 Canonical Ltd.
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; version 3 of the License.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.


from .base_task import ContainerBaseTask
from libertine import LibertineContainer, utils


class DestroyTask(ContainerBaseTask):
    def __init__(self, container_id, config, force, lock, monitor, client):
        super().__init__(lock=lock, container_id=container_id, config=config,
                         monitor=monitor, client=client)
        self._force = force

    def _run(self):
        utils.get_logger().debug("Destroying container '%s'" % self._container)

        container = LibertineContainer(self._container, self._config, self._client)
        if not container.destroy_libertine_container(self._force):
            self._error("Destroying container '%s' failed" % self._container)
            self._config.update_container_install_status(self._container, self._fallback)
            return

        self._config.update_container_install_status(self._container, "removed")
        self._config.delete_container(self._container)

        utils.refresh_libertine_scope()

    def _before(self):
        utils.get_logger().debug("DestroyTask::_before")
        self._fallback = self._config.get_container_install_status(self._container)
        self._config.update_container_install_status(self._container, 'removing')
        return True
